package org.grails.twitter

import org.grails.twitter.auth.Person

import grails.plugins.springsecurity.Secured

import net.sf.ehcache.Element

@Secured(['IS_AUTHENTICATED_FULLY'])
class StatusController {

	def springSecurityService
	def statusService
	def twitterCache

	def index() {
		[messages: currentUserTimeline()]
	}

	def updateStatus = {

		statusService.sendQueueJMSMessage('queue.status',
			[personId: springSecurityService.principal.id,
          username: springSecurityService.principal.username,
          message: params.message])

		def messages = currentUserTimeline()
		render template: 'messages', collection: messages, var: 'message'
	}

	private lookupPerson() {
		Person.load(springSecurityService.principal.id)
	}

	private currentUserTimeline() {
		String username = springSecurityService.principal.username
		def messages = twitterCache.get(username)
		if (!messages) {
			log.info "No messages found in cache for user <${username}>. Querying database..."
			messages = Status.withCriteria {
				def currentUser = lookupPerson()
				or {
					author {
						eq 'username', username
					}
					if (currentUser.followed) {
						inList 'author', currentUser.followed
					}
				}
				maxResults 10
				order 'dateCreated', 'desc'
			}

			twitterCache.put new Element(username, messages)
		}
		else {
			messages = messages.value
			log.info "Status messages loaded from cache for user <${username}>."
		}
		messages
	}

	def follow() {
		def person = Person.get(params.id)
		if (person) {
			def currentUser = lookupPerson()
			currentUser.addToFollowed person
			currentUser.save()
			twitterCache.remove springSecurityService.principal.username
		}
		redirect action: 'index'
	}
}

